@@ -12,6 +12,8 @@ if (typeof(KISMET_URI_PREFIX) !== 'undefined')
 
 var exports = {};
 
+exports.window_visible = true;
+
 // Load spectrum css and js
 $('<link>')
     .appendTo('head')
@@ -146,6 +148,9 @@ function deviceview_selector_dynamic_update() {
     clearTimeout(view_list_updater_tid);
     view_list_updater_tid = setTimeout(deviceview_selector_dynamic_update, 5000);
 
+    if (!exports.window_visible)
+        return;
+
     var ds_priority = -5000;
     var phy_priority = -1000;
 
@@ -161,8 +166,6 @@ function deviceview_selector_dynamic_update() {
                     var dsdata = kismet.sanitizeObject(dsdata);
                     var synth_view = 'seenby-' + dsdata['kismet.datasource.uuid'];
 
-                    console.log(synth_view);
-
                     existing_views[synth_view] = 1;
 
                     exports.AddDeviceView(dsdata['kismet.datasource.name'], synth_view, ds_priority, 'Datasources');
@@ -223,7 +226,8 @@ var DeviceRowHighlights = new Array();
  * name: datatable 'name' field (optional)
  * field: Kismet field path, array pair of field path and name, array of fields,
  *  or a function returning one of the above.
- * fields: Multiple fields
+ * fields: Multiple fields.  When multiple fields are defined, ONE field MUST be defined in the
+ *  'field' parameter.  Additional multiple fields may be defined in this parameter.
  * renderfunc: string name of datatable render function, taking DT arguments
  *  (data, type, row, meta), (optional)
  * drawfunc: string name of a draw function, taking arguments:
@@ -481,13 +485,11 @@ exports.GetDeviceFields = function(selected) {
     var cols = exports.GetDeviceColumns();
 
     for (var i in cols) {
-        if ('field' in cols[i]) {
+        if ('field' in cols[i] && cols[i]['field'] != null) 
             rawret.push(cols[i]['field']);
-        }
 
-        if ('fields' in cols[i]) {
-            rawret.push.apply(rawret, cols[i]['fields']);
-        }
+        if ('fields' in cols[i] && cols[i]['fields'] != null) 
+            cols[i]['fields'].forEach(function(e) { rawret.push.apply(rawret, e); });
     }
 
     for (var i in DeviceRowHighlights) {
@@ -502,29 +504,7 @@ exports.GetDeviceFields = function(selected) {
     return ret;
 }
 
-exports.DeviceDetails = new Array();
-
-/* Register a device detail accordion panel, taking an id for the panel
- * content, a title presented to the user, a position in the list, and
- * options.  Because details are directly rendered all the time and
- * can't be moved around / saved as configs like columns can, callbacks
- * are just direct functions here.
- *
- * filter and render take one argument, the data to be shown
- * filter: function(data) {
- *  return false;
- * }
- *
- * render: function(data) {
- *  return "Some content";
- * }
- *
- * draw takes the device data and a container element as an argument:
- * draw: function(data, element) {
- *  e.append("hi");
- * }
- * */
-exports.AddDeviceDetail = function(id, title, pos, options) {
+exports.AddDetail = function(container, id, title, pos, options) {
     var settings = $.extend({
         "filter": null,
         "render": null,
@@ -538,20 +518,16 @@ exports.AddDeviceDetail = function(id, title, pos, options) {
         options: settings
     };
 
-    exports.DeviceDetails.push(det);
+    container.push(det);
 
-    exports.DeviceDetails.sort(function(a, b) {
+    container.sort(function(a, b) {
         return a.position - b.position;
     });
 }
 
-exports.GetDeviceDetails = function() {
-    return exports.DeviceDetails;
-}
-
-exports.DeviceDetailWindow = function(key) {
+exports.DetailWindow = function(key, title, options, window_cb, close_cb) {
     // Generate a unique ID for this dialog
-    var dialogid = "devicedialog" + key;
+    var dialogid = "detailialog" + key;
     var dialogmatch = '#' + dialogid;
 
     if (jsPanel.activePanels.list.indexOf(dialogid) != -1) {
@@ -577,7 +553,7 @@ exports.DeviceDetailWindow = function(key) {
 
     var panel = $.jsPanel({
         id: dialogid,
-        headerTitle: 'Device Details',
+        headerTitle: title,
 
         headerControls: {
             iconfont: 'jsglyph',
@@ -595,7 +571,6 @@ exports.DeviceDetailWindow = function(key) {
 
         resizable: {
             minWidth: 450,
-            maxWidth: 600,
             minHeight: 400,
             stop: function(event, ui) {
                 $('div#accordion', ui.element).accordion("refresh");
@@ -610,66 +585,13 @@ exports.DeviceDetailWindow = function(key) {
             $('div#accordion', this.content).accordion("refresh");
         },
 
+        onclosed: function() {
+            close_cb(this, options);
+        },
+
         callback: function() {
-            var panel = this;
-            var content = this.content;
-
-            $.get(local_uri_prefix + "devices/by-key/" + key + "/device.json")
-                .done(function(fulldata) {
-                    fulldata = kismet.sanitizeObject(fulldata);
-
-                    panel.headerTitle(fulldata['kismet_device_base_name']);
-
-                    var accordion = $('<div />', {
-                        id: 'accordion'
-                    });
-
-                    content.append(accordion);
-
-                    var detailslist = kismet_ui.GetDeviceDetails();
-
-                    for (var dii in detailslist) {
-                        var di = detailslist[dii];
-
-                        // Do we skip?
-                        if ('filter' in di.options &&
-                                typeof(di.options.filter) === 'function') {
-                            if (di.options.filter(fulldata) == false) {
-                                continue;
-                            }
-                        }
-
-                        var vheader = $('<h3 />', {
-                            id: "header" + di.id,
-                            html: di.title
-                        });
-
-                        var vcontent = $('<div />', {
-                            id: di.id,
-                            //class: 'autosize'
-                        });
-
-                        // Do we have pre-rendered content?
-                        if ('render' in di.options &&
-                                typeof(di.options.render) === 'function') {
-                            vcontent.html(di.options.render(fulldata));
-                        }
-
-                        accordion.append(vheader);
-                        accordion.append(vcontent);
-
-                        if ('draw' in di.options &&
-                                typeof(di.options.draw) === 'function') {
-                            di.options.draw(fulldata, vcontent);
-                        }
-                    }
-                    accordion.accordion({ heightStyle: 'fill' });
-                })
-            .fail(function(jqxhr, texterror) {
-                content.html("<div style=\"padding: 10px;\"><h1>Oops!</h1><p>An error occurred loading device details for key <code>" + key + 
-                        "</code>: HTTP code <code>" + jqxhr.status + "</code>, " + texterror + "</div>");
-            });
-        }
+            window_cb(this, options);
+        },
     }).resize({
         width: w,
         height: h,
@@ -690,6 +612,145 @@ exports.DeviceDetailWindow = function(key) {
             "offsetY": 2,
         });
     }
+
+}
+
+exports.DeviceDetails = new Array();
+
+/* Register a device detail accordion panel, taking an id for the panel
+ * content, a title presented to the user, a position in the list, and
+ * options.  Because details are directly rendered all the time and
+ * can't be moved around / saved as configs like columns can, callbacks
+ * are just direct functions here.
+ *
+ * filter and render take one argument, the data to be shown
+ * filter: function(data) {
+ *  return false;
+ * }
+ *
+ * render: function(data) {
+ *  return "Some content";
+ * }
+ *
+ * draw takes the device data and a container element as an argument:
+ * draw: function(data, element) {
+ *  e.append("hi");
+ * }
+ * */
+exports.AddDeviceDetail = function(id, title, pos, options) {
+    exports.AddDetail(exports.DeviceDetails, id, title, pos, options);
+}
+
+exports.GetDeviceDetails = function() {
+    return exports.DeviceDetails;
+}
+
+exports.DeviceDetailWindow = function(key) {
+    exports.DetailWindow(key, "Device Details", 
+        {
+            storage: {}
+        },
+
+        function(panel, options) {
+            var content = panel.content;
+
+            panel.active = true;
+
+            window['storage_devlist_' + key] = {};
+
+            window['storage_devlist_' + key]['foobar'] = 'bar';
+
+            panel.updater = function() {
+                if (exports.window_visible) {
+                    $.get(local_uri_prefix + "devices/by-key/" + key + "/device.json")
+                        .done(function(fulldata) {
+                            fulldata = kismet.sanitizeObject(fulldata);
+
+                            panel.headerTitle("Device: " + kismet.censorMAC(fulldata['kismet.device.base.commonname']));
+
+                            var accordion = $('div#accordion', content);
+
+                            if (accordion.length == 0) {
+                                accordion = $('<div />', {
+                                    id: 'accordion'
+                                });
+
+                                content.append(accordion);
+                            }
+
+                            var detailslist = kismet_ui.GetDeviceDetails();
+
+                            for (var dii in detailslist) {
+                                var di = detailslist[dii];
+
+                                // Do we skip?
+                                if ('filter' in di.options &&
+                                    typeof(di.options.filter) === 'function') {
+                                    if (di.options.filter(fulldata) == false) {
+                                        continue;
+                                    }
+                                }
+
+                                var vheader = $('h3#header_' + di.id, accordion);
+
+                                if (vheader.length == 0) {
+                                    vheader = $('<h3>', {
+                                        id: "header_" + di.id,
+                                    })
+                                        .html(di.title);
+
+                                    accordion.append(vheader);
+                                }
+
+                                var vcontent = $('div#' + di.id, accordion);
+
+                                if (vcontent.length == 0) {
+                                    vcontent = $('<div>', {
+                                        id: di.id,
+                                    });
+                                    accordion.append(vcontent);
+                                }
+
+                                // Do we have pre-rendered content?
+                                if ('render' in di.options &&
+                                    typeof(di.options.render) === 'function') {
+                                    vcontent.html(di.options.render(fulldata));
+                                }
+
+                                if ('draw' in di.options &&
+                                    typeof(di.options.draw) === 'function') {
+                                    di.options.draw(fulldata, vcontent, options, 'storage_devlist_' + key);
+                                }
+
+                                if ('finalize' in di.options &&
+                                    typeof(di.options.finalize) === 'function') {
+                                    di.options.finalize(fulldata, vcontent, options, 'storage_devlist_' + key);
+                                }
+                            }
+                            accordion.accordion({ heightStyle: 'fill' });
+                        })
+                        .fail(function(jqxhr, texterror) {
+                            content.html("<div style=\"padding: 10px;\"><h1>Oops!</h1><p>An error occurred loading device details for key <code>" + key + 
+                                "</code>: HTTP code <code>" + jqxhr.status + "</code>, " + texterror + "</div>");
+                        })
+                        .always(function() {
+                            panel.timerid = setTimeout(function() { panel.updater(); }, 1000);
+                        })
+                } else {
+                    panel.timerid = setTimeout(function() { panel.updater(); }, 1000);
+                }
+
+            };
+
+            panel.updater();
+        },
+
+        function(panel, options) {
+            clearTimeout(panel.timerid);
+            panel.active = false;
+            window['storage_devlist_' + key] = {};
+        });
+
 };
 
 exports.RenderTrimmedTime = function(opts) {
@@ -757,41 +818,48 @@ exports.connection_error_panel = null;
 exports.HealthCheck = function() {
     var timerid;
 
-    $.get(local_uri_prefix + "system/status.json")
-    .done(function(data) {
-        data = kismet.sanitizeObject(data);
+    if (exports.window_visible) {
+        $.get(local_uri_prefix + "system/status.json")
+            .done(function(data) {
+                data = kismet.sanitizeObject(data);
 
-        if (exports.connection_error) {
-            exports.connection_error_panel.close();
-        }
+                if (exports.connection_error) {
+                    exports.connection_error_panel.close();
+                }
 
-        exports.connection_error = false;
+                exports.connection_error = false;
 
-        exports.last_timestamp = data['kismet.system.timestamp.sec'];
-    })
-    .fail(function() {
-        if (!exports.connection_error) {
-            exports.connection_error_panel = $.jsPanel({
-                id: "connection-alert",
-                headerTitle: 'Cannot Connect to Kismet',
-                headerControls: {
-                    controls: 'none',
-                    iconfont: 'jsglyph',
-                },
-                contentSize: "auto auto",
-                paneltype: 'modal',
-                content: '<div style="padding: 10px;"><h3><i class="fa fa-exclamation-triangle" style="color: red;" /> Sorry!</h3><p>Cannot connect to the Kismet webserver.  Make sure Kismet is still running on this host!<p><i class="fa fa-refresh fa-spin" style="margin-right: 5px" /> Connecting to the Kismet server...</div>',
-            });
-        }
+                exports.last_timestamp = data['kismet.system.timestamp.sec'];
+            })
+            .fail(function() {
+                if (!exports.connection_error) {
+                    exports.connection_error_panel = $.jsPanel({
+                        id: "connection-alert",
+                        headerTitle: 'Cannot Connect to Kismet',
+                        headerControls: {
+                            controls: 'none',
+                            iconfont: 'jsglyph',
+                        },
+                        contentSize: "auto auto",
+                        paneltype: 'modal',
+                        content: '<div style="padding: 10px;"><h3><i class="fa fa-exclamation-triangle" style="color: red;" /> Sorry!</h3><p>Cannot connect to the Kismet webserver.  Make sure Kismet is still running on this host!<p><i class="fa fa-refresh fa-spin" style="margin-right: 5px" /> Connecting to the Kismet server...</div>',
+                    });
+                }
 
-        exports.connection_error = true;
-    })
-    .always(function() {
+                exports.connection_error = true;
+            })
+            .always(function() {
+                if (exports.connection_error)
+                    timerid = setTimeout(exports.HealthCheck, 1000);
+                else
+                    timerid = setTimeout(exports.HealthCheck, 5000);
+            }); 
+    } else {
         if (exports.connection_error)
             timerid = setTimeout(exports.HealthCheck, 1000);
         else
             timerid = setTimeout(exports.HealthCheck, 5000);
-    });
+    }
 
 }
 
@@ -877,25 +945,35 @@ exports.renderTemperature = function(c, precision = 5) {
 
 var deviceTid;
 
+var devicetableElement = null;
+
 function ScheduleDeviceSummary() {
-    var dt = $('#devices').DataTable();
+    try {
+        if (exports.window_visible && devicetableElement.is(":visible")) {
 
-    // Save the state.  We can't use proper state saving because it seems to break
-    // the table position
-    kismet.putStorage('kismet.base.devicetable.order', JSON.stringify(dt.order()));
-    kismet.putStorage('kismet.base.devicetable.search', JSON.stringify(dt.search()));
+            var dt = devicetableElement.DataTable();
 
-    // Snapshot where we are, because the 'don't reset page' in ajax.reload
-    // DOES still reset the scroll position
-    var prev_pos = {
-        'top': $(dt.settings()[0].nScrollBody).scrollTop(),
-        'left': $(dt.settings()[0].nScrollBody).scrollLeft()
-    };
-    dt.ajax.reload(function(d) {
-            // Restore our scroll position
-            $(dt.settings()[0].nScrollBody).scrollTop( prev_pos.top );
-            $(dt.settings()[0].nScrollBody).scrollLeft( prev_pos.left );
-        }, false);
+            // Save the state.  We can't use proper state saving because it seems to break
+            // the table position
+            kismet.putStorage('kismet.base.devicetable.order', JSON.stringify(dt.order()));
+            kismet.putStorage('kismet.base.devicetable.search', JSON.stringify(dt.search()));
+
+            // Snapshot where we are, because the 'don't reset page' in ajax.reload
+            // DOES still reset the scroll position
+            var prev_pos = {
+                'top': $(dt.settings()[0].nScrollBody).scrollTop(),
+                'left': $(dt.settings()[0].nScrollBody).scrollLeft()
+            };
+            dt.ajax.reload(function(d) {
+                // Restore our scroll position
+                $(dt.settings()[0].nScrollBody).scrollTop( prev_pos.top );
+                $(dt.settings()[0].nScrollBody).scrollLeft( prev_pos.left );
+            }, false);
+        }
+
+    } catch (error) {
+        ;
+    }
     
     // Set our timer outside of the datatable callback so that we get called even
     // if the ajax load fails
@@ -908,8 +986,6 @@ function CancelDeviceSummary() {
     clearTimeout(deviceTid);
 }
 
-var devicetableElement = null;
-
 /* Create the device table */
 exports.CreateDeviceTable = function(element) {
     devicetableElement = element;
@@ -917,35 +993,6 @@ exports.CreateDeviceTable = function(element) {
 
     var dt = exports.InitializeDeviceTable(element);
 
-    // Set an onclick handler to spawn the device details dialog
-    $('tbody', element).on('click', 'tr', function () {
-        kismet_ui.DeviceDetailWindow(this.id);
-
-        // Use the ID above we insert in the row creation, instead of looking in the
-        // device list data
-        // Fetch the data of the row that got clicked
-        // var device_dt = element.DataTable();
-        // var data = device_dt.row( this ).data();
-        // var key = data['kismet.device.base.key'];
-        // kismet_ui.DeviceDetailWindow(key);
-    } );
-
-    $('tbody', element)
-        .on( 'mouseenter', 'td', function () {
-            var device_dt = element.DataTable();
-
-            if (typeof(device_dt.cell(this).index()) === 'Undefined')
-                return;
-
-            var colIdx = device_dt.cell(this).index().column;
-            var rowIdx = device_dt.cell(this).index().row;
-
-            // Remove from all cells
-            $(device_dt.cells().nodes()).removeClass('kismet-highlight');
-            // Highlight the td in this row
-            $('td', device_dt.row(rowIdx).nodes()).addClass('kismet-highlight');
-        } );
-
     dt.draw(false);
 
     // Start the auto-updating
@@ -1043,17 +1090,25 @@ exports.InitializeDeviceTable = function(element) {
                     }
 
                     // Call the draw callback if one exists
-                    col.kismetdrawfunc(col, dt, this);
+                    try {
+                        col.kismetdrawfunc(col, dt, this);
+                    } catch (error) {
+                        ;
+                    }
                 }
 
                 for (var r in DeviceRowHighlights) {
-                    var rowh = DeviceRowHighlights[r];
+                    try {
+                        var rowh = DeviceRowHighlights[r];
 
-                    if (rowh['enable']) {
-                        if (rowh['selector'](this.data())) {
-                            $('td', this.node()).css('background-color', rowh['color']);
-                            break;
+                        if (rowh['enable']) {
+                            if (rowh['selector'](this.data())) {
+                                $('td', this.node()).css('background-color', rowh['color']);
+                                break;
+                            }
                         }
+                    } catch (error) {
+                        ;
                     }
                 }
             }
@@ -1078,6 +1133,36 @@ exports.InitializeDeviceTable = function(element) {
     if (saved_search !== "")
         device_dt.search(JSON.parse(saved_search));
 
+    // Set an onclick handler to spawn the device details dialog
+    $('tbody', element).on('click', 'tr', function () {
+        kismet_ui.DeviceDetailWindow(this.id);
+
+        // Use the ID above we insert in the row creation, instead of looking in the
+        // device list data
+        // Fetch the data of the row that got clicked
+        // var device_dt = element.DataTable();
+        // var data = device_dt.row( this ).data();
+        // var key = data['kismet.device.base.key'];
+        // kismet_ui.DeviceDetailWindow(key);
+    } );
+
+    $('tbody', element)
+        .on( 'mouseenter', 'td', function () {
+            var device_dt = element.DataTable();
+
+            if (typeof(device_dt.cell(this).index()) === 'Undefined')
+                return;
+
+            var colIdx = device_dt.cell(this).index().column;
+            var rowIdx = device_dt.cell(this).index().row;
+
+            // Remove from all cells
+            $(device_dt.cells().nodes()).removeClass('kismet-highlight');
+            // Highlight the td in this row
+            $('td', device_dt.row(rowIdx).nodes()).addClass('kismet-highlight');
+        } );
+
+
     return device_dt;
 }
 
